/*
 * The Npic library
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/* DO NOT EDIT !!!  Generated by npic-templa from "files_raw.ct" */

/*
 * files_raw.c - 17/12/2008
 *
 * Raw image files.
*/


#include <npic.h>

#define FBUF_INT8   4096
#define FBUF_INT16  2048
#define FBUF_INT32  1024
#define FBUF_DOUBLE  512


/*--------------------- P U B L I C - I N T E R F A C E ----------------------*/


/*-------------------- P R I V A T E - F U N C T I O N S ---------------------*/

/*
 * Write file f in RAW binary format.
 * Only interior pixels are saved.
 * Return NPIC_SUCCESS, else error code. Silent.
*/

int NpicWriteRAW (Npic_image *np, FILE *f, int bswap)
{
    if (np == NULL) return NPIC_ERR_NULL_PTR;
    if (f == NULL) return NPIC_ERR_BAD_FD;

    switch (np->type) {

        case NPIC_IMAGE_2C : {
            int y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_2c *I = NpicCastImage(np);

            j = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[y][x];
                if (j >= FBUF_INT8) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_2L : {
            int y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_2l *I = NpicCastImage(np);

            j = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = bswap ? NPIC_BSWAP32 (I->pix[y][x]) : I->pix[y][x];
                if (j >= FBUF_INT32) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_2D : {
            int y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_2d *I = NpicCastImage(np);

            j = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[y][x];
                if (j >= FBUF_DOUBLE) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_2Q : {
            int y, x, j, k;
            Npic_uint16 buf[FBUF_INT16+4];
            Npic_image_2q *I = NpicCastImage(np);

            j = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (bswap) {
                    buf[j++] = NPIC_BSWAP16 (I->pix[y][x].a);
                    buf[j++] = NPIC_BSWAP16 (I->pix[y][x].b);
                    buf[j++] = NPIC_BSWAP16 (I->pix[y][x].c);
                    buf[j++] = NPIC_BSWAP16 (I->pix[y][x].d);
                } else {
                    buf[j++] = I->pix[y][x].a;
                    buf[j++] = I->pix[y][x].b;
                    buf[j++] = I->pix[y][x].c;
                    buf[j++] = I->pix[y][x].d;
                }
                if (j >= FBUF_INT16) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_3C : {
            int z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_3c *I = NpicCastImage(np);

            j = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[z][y][x];
                if (j >= FBUF_INT8) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_3L : {
            int z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_3l *I = NpicCastImage(np);

            j = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = bswap ? NPIC_BSWAP32 (I->pix[z][y][x]) : I->pix[z][y][x];
                if (j >= FBUF_INT32) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_3D : {
            int z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_3d *I = NpicCastImage(np);

            j = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[z][y][x];
                if (j >= FBUF_DOUBLE) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_3Q : {
            int z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16+4];
            Npic_image_3q *I = NpicCastImage(np);

            j = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (bswap) {
                    buf[j++] = NPIC_BSWAP16 (I->pix[z][y][x].a);
                    buf[j++] = NPIC_BSWAP16 (I->pix[z][y][x].b);
                    buf[j++] = NPIC_BSWAP16 (I->pix[z][y][x].c);
                    buf[j++] = NPIC_BSWAP16 (I->pix[z][y][x].d);
                } else {
                    buf[j++] = I->pix[z][y][x].a;
                    buf[j++] = I->pix[z][y][x].b;
                    buf[j++] = I->pix[z][y][x].c;
                    buf[j++] = I->pix[z][y][x].d;
                }
                if (j >= FBUF_INT16) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_4C : {
            int t, z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_4c *I = NpicCastImage(np);

            j = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[t][z][y][x];
                if (j >= FBUF_INT8) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_4L : {
            int t, z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_4l *I = NpicCastImage(np);

            j = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = bswap ? NPIC_BSWAP32 (I->pix[t][z][y][x]) : I->pix[t][z][y][x];
                if (j >= FBUF_INT32) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_4D : {
            int t, z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_4d *I = NpicCastImage(np);

            j = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[t][z][y][x];
                if (j >= FBUF_DOUBLE) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_4Q : {
            int t, z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16+4];
            Npic_image_4q *I = NpicCastImage(np);

            j = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (bswap) {
                    buf[j++] = NPIC_BSWAP16 (I->pix[t][z][y][x].a);
                    buf[j++] = NPIC_BSWAP16 (I->pix[t][z][y][x].b);
                    buf[j++] = NPIC_BSWAP16 (I->pix[t][z][y][x].c);
                    buf[j++] = NPIC_BSWAP16 (I->pix[t][z][y][x].d);
                } else {
                    buf[j++] = I->pix[t][z][y][x].a;
                    buf[j++] = I->pix[t][z][y][x].b;
                    buf[j++] = I->pix[t][z][y][x].c;
                    buf[j++] = I->pix[t][z][y][x].d;
                }
                if (j >= FBUF_INT16) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_5C : {
            int s, t, z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_5c *I = NpicCastImage(np);

            j = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[s][t][z][y][x];
                if (j >= FBUF_INT8) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_5L : {
            int s, t, z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_5l *I = NpicCastImage(np);

            j = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = bswap ? NPIC_BSWAP32 (I->pix[s][t][z][y][x]) : I->pix[s][t][z][y][x];
                if (j >= FBUF_INT32) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_5D : {
            int s, t, z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_5d *I = NpicCastImage(np);

            j = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[s][t][z][y][x];
                if (j >= FBUF_DOUBLE) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_5Q : {
            int s, t, z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16+4];
            Npic_image_5q *I = NpicCastImage(np);

            j = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (bswap) {
                    buf[j++] = NPIC_BSWAP16 (I->pix[s][t][z][y][x].a);
                    buf[j++] = NPIC_BSWAP16 (I->pix[s][t][z][y][x].b);
                    buf[j++] = NPIC_BSWAP16 (I->pix[s][t][z][y][x].c);
                    buf[j++] = NPIC_BSWAP16 (I->pix[s][t][z][y][x].d);
                } else {
                    buf[j++] = I->pix[s][t][z][y][x].a;
                    buf[j++] = I->pix[s][t][z][y][x].b;
                    buf[j++] = I->pix[s][t][z][y][x].c;
                    buf[j++] = I->pix[s][t][z][y][x].d;
                }
                if (j >= FBUF_INT16) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_6C : {
            int r, s, t, z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_6c *I = NpicCastImage(np);

            j = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[r][s][t][z][y][x];
                if (j >= FBUF_INT8) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_6L : {
            int r, s, t, z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_6l *I = NpicCastImage(np);

            j = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = bswap ? NPIC_BSWAP32 (I->pix[r][s][t][z][y][x]) : I->pix[r][s][t][z][y][x];
                if (j >= FBUF_INT32) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_6D : {
            int r, s, t, z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_6d *I = NpicCastImage(np);

            j = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                buf[j++] = I->pix[r][s][t][z][y][x];
                if (j >= FBUF_DOUBLE) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        case NPIC_IMAGE_6Q : {
            int r, s, t, z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16+4];
            Npic_image_6q *I = NpicCastImage(np);

            j = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (bswap) {
                    buf[j++] = NPIC_BSWAP16 (I->pix[r][s][t][z][y][x].a);
                    buf[j++] = NPIC_BSWAP16 (I->pix[r][s][t][z][y][x].b);
                    buf[j++] = NPIC_BSWAP16 (I->pix[r][s][t][z][y][x].c);
                    buf[j++] = NPIC_BSWAP16 (I->pix[r][s][t][z][y][x].d);
                } else {
                    buf[j++] = I->pix[r][s][t][z][y][x].a;
                    buf[j++] = I->pix[r][s][t][z][y][x].b;
                    buf[j++] = I->pix[r][s][t][z][y][x].c;
                    buf[j++] = I->pix[r][s][t][z][y][x].d;
                }
                if (j >= FBUF_INT16) {
                    k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                    if (k != 1) return NPIC_ERR_WRITE;
                    j = 0;
                }
            }
            
            /* Flush buf */
            if (j > 0) {
                k = fwrite (buf, sizeof(buf[0])*j, 1, f);
                if (k != 1) return NPIC_ERR_WRITE;
            }
        } break;

        default : return NPIC_ERR_UNEX_NPIC;
    }

    return NPIC_SUCCESS;
}


/*
 * Read file f in RAW binary format.
 * Byteswap pixels if bswap == 1.
 * Return NPIC_SUCCESS, else error code. Silent.
*/

int NpicReadRAW  (Npic_image *np, FILE *f, int bswap)
{
    if (np == NULL) return NPIC_ERR_NULL_PTR;
    if (f == NULL) return NPIC_ERR_BAD_FD;

    switch (np->type) {

        case NPIC_IMAGE_2C : {
            int y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_2c *I = NpicCastImage(np);

            j = k = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT8, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[y][x] = buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_2L : {
            int y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_2l *I = NpicCastImage(np);

            j = k = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT32, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[y][x] = bswap ? NPIC_BSWAP32(buf[j]) : buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_2D : {
            int y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_2d *I = NpicCastImage(np);
            union { Npic_d f; long long i; } u;

            j = k = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_DOUBLE, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                u.f = buf[j]; if (bswap) u.i = NPIC_BSWAP64(u.i);
                I->pix[y][x] = u.f; j++;
            }
        } break;

        case NPIC_IMAGE_2Q : {
            int y, x, j, k;
            Npic_uint16 buf[FBUF_INT16];
            Npic_image_2q *I = NpicCastImage(np);

            j = k = 0;
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0])*4, FBUF_INT16/4, f)*4;
                    if (k <= 0) return NPIC_ERR_READ;
                }
                if (bswap) {
                    I->pix[y][x].a = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[y][x].b = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[y][x].c = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[y][x].d = NPIC_BSWAP16 (buf[j]); j++;
                } else {
                    I->pix[y][x].a = buf[j++];
                    I->pix[y][x].b = buf[j++];
                    I->pix[y][x].c = buf[j++];
                    I->pix[y][x].d = buf[j++];
                }
            }
        } break;

        case NPIC_IMAGE_3C : {
            int z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_3c *I = NpicCastImage(np);

            j = k = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT8, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[z][y][x] = buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_3L : {
            int z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_3l *I = NpicCastImage(np);

            j = k = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT32, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[z][y][x] = bswap ? NPIC_BSWAP32(buf[j]) : buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_3D : {
            int z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_3d *I = NpicCastImage(np);
            union { Npic_d f; long long i; } u;

            j = k = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_DOUBLE, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                u.f = buf[j]; if (bswap) u.i = NPIC_BSWAP64(u.i);
                I->pix[z][y][x] = u.f; j++;
            }
        } break;

        case NPIC_IMAGE_3Q : {
            int z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16];
            Npic_image_3q *I = NpicCastImage(np);

            j = k = 0;
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0])*4, FBUF_INT16/4, f)*4;
                    if (k <= 0) return NPIC_ERR_READ;
                }
                if (bswap) {
                    I->pix[z][y][x].a = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[z][y][x].b = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[z][y][x].c = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[z][y][x].d = NPIC_BSWAP16 (buf[j]); j++;
                } else {
                    I->pix[z][y][x].a = buf[j++];
                    I->pix[z][y][x].b = buf[j++];
                    I->pix[z][y][x].c = buf[j++];
                    I->pix[z][y][x].d = buf[j++];
                }
            }
        } break;

        case NPIC_IMAGE_4C : {
            int t, z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_4c *I = NpicCastImage(np);

            j = k = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT8, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[t][z][y][x] = buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_4L : {
            int t, z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_4l *I = NpicCastImage(np);

            j = k = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT32, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[t][z][y][x] = bswap ? NPIC_BSWAP32(buf[j]) : buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_4D : {
            int t, z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_4d *I = NpicCastImage(np);
            union { Npic_d f; long long i; } u;

            j = k = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_DOUBLE, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                u.f = buf[j]; if (bswap) u.i = NPIC_BSWAP64(u.i);
                I->pix[t][z][y][x] = u.f; j++;
            }
        } break;

        case NPIC_IMAGE_4Q : {
            int t, z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16];
            Npic_image_4q *I = NpicCastImage(np);

            j = k = 0;
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0])*4, FBUF_INT16/4, f)*4;
                    if (k <= 0) return NPIC_ERR_READ;
                }
                if (bswap) {
                    I->pix[t][z][y][x].a = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[t][z][y][x].b = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[t][z][y][x].c = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[t][z][y][x].d = NPIC_BSWAP16 (buf[j]); j++;
                } else {
                    I->pix[t][z][y][x].a = buf[j++];
                    I->pix[t][z][y][x].b = buf[j++];
                    I->pix[t][z][y][x].c = buf[j++];
                    I->pix[t][z][y][x].d = buf[j++];
                }
            }
        } break;

        case NPIC_IMAGE_5C : {
            int s, t, z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_5c *I = NpicCastImage(np);

            j = k = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT8, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[s][t][z][y][x] = buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_5L : {
            int s, t, z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_5l *I = NpicCastImage(np);

            j = k = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT32, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[s][t][z][y][x] = bswap ? NPIC_BSWAP32(buf[j]) : buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_5D : {
            int s, t, z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_5d *I = NpicCastImage(np);
            union { Npic_d f; long long i; } u;

            j = k = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_DOUBLE, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                u.f = buf[j]; if (bswap) u.i = NPIC_BSWAP64(u.i);
                I->pix[s][t][z][y][x] = u.f; j++;
            }
        } break;

        case NPIC_IMAGE_5Q : {
            int s, t, z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16];
            Npic_image_5q *I = NpicCastImage(np);

            j = k = 0;
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0])*4, FBUF_INT16/4, f)*4;
                    if (k <= 0) return NPIC_ERR_READ;
                }
                if (bswap) {
                    I->pix[s][t][z][y][x].a = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[s][t][z][y][x].b = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[s][t][z][y][x].c = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[s][t][z][y][x].d = NPIC_BSWAP16 (buf[j]); j++;
                } else {
                    I->pix[s][t][z][y][x].a = buf[j++];
                    I->pix[s][t][z][y][x].b = buf[j++];
                    I->pix[s][t][z][y][x].c = buf[j++];
                    I->pix[s][t][z][y][x].d = buf[j++];
                }
            }
        } break;

        case NPIC_IMAGE_6C : {
            int r, s, t, z, y, x, j, k;
            Npic_c buf[FBUF_INT8];
            Npic_image_6c *I = NpicCastImage(np);

            j = k = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT8, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[r][s][t][z][y][x] = buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_6L : {
            int r, s, t, z, y, x, j, k;
            Npic_l buf[FBUF_INT32];
            Npic_image_6l *I = NpicCastImage(np);

            j = k = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_INT32, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                I->pix[r][s][t][z][y][x] = bswap ? NPIC_BSWAP32(buf[j]) : buf[j]; j++;
            }
        } break;

        case NPIC_IMAGE_6D : {
            int r, s, t, z, y, x, j, k;
            Npic_d buf[FBUF_DOUBLE];
            Npic_image_6d *I = NpicCastImage(np);
            union { Npic_d f; long long i; } u;

            j = k = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0]), FBUF_DOUBLE, f);
                    if (k <= 0) return NPIC_ERR_READ;
                }
                u.f = buf[j]; if (bswap) u.i = NPIC_BSWAP64(u.i);
                I->pix[r][s][t][z][y][x] = u.f; j++;
            }
        } break;

        case NPIC_IMAGE_6Q : {
            int r, s, t, z, y, x, j, k;
            Npic_uint16 buf[FBUF_INT16];
            Npic_image_6q *I = NpicCastImage(np);

            j = k = 0;
            for (r = 0; r < I->rmax; r++)
            for (s = 0; s < I->smax; s++)
            for (t = 0; t < I->tmax; t++)
            for (z = 0; z < I->zmax; z++)
            for (y = 0; y < I->ymax; y++)
            for (x = 0; x < I->xmax; x++)
            {
                if (j >= k) {
                    j = 0;
                    k = fread (buf, sizeof(buf[0])*4, FBUF_INT16/4, f)*4;
                    if (k <= 0) return NPIC_ERR_READ;
                }
                if (bswap) {
                    I->pix[r][s][t][z][y][x].a = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[r][s][t][z][y][x].b = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[r][s][t][z][y][x].c = NPIC_BSWAP16 (buf[j]); j++;
                    I->pix[r][s][t][z][y][x].d = NPIC_BSWAP16 (buf[j]); j++;
                } else {
                    I->pix[r][s][t][z][y][x].a = buf[j++];
                    I->pix[r][s][t][z][y][x].b = buf[j++];
                    I->pix[r][s][t][z][y][x].c = buf[j++];
                    I->pix[r][s][t][z][y][x].d = buf[j++];
                }
            }
        } break;

        default : return NPIC_ERR_UNEX_NPIC;
    }
    
    return NPIC_SUCCESS;
}


/*----------------------------------------------------------------------------*/


