/*
 * The Npic library
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/*
 * mlut_ma.h - 12/11/2008
 *
 * Computation of Medial Axis on Distance Transforms using LUTs.
*/


#ifndef NPIC__MLUT_MA_H
#define NPIC__MLUT_MA_H


/* PUBLIC */

void NpicExtractMA_bin (Npic_image *np1, Npic_image *nMA,
        Npic_mask *nMdist, Npic_mask *nMlut);
void NpicExtractMA_DT (Npic_image *nDT, Npic_image *nMA,
        Npic_mask *nMdist, Npic_mask *nMlut);
void NpicExtractMA_Lut (Npic_image *nDT, Npic_image *nMA,
        Npic_mask *nMlut, Npic_Lut *Lut, Npic_l Rmax);
void NpicExtractMA_Mlut (Npic_image *nDT, Npic_image *nMA, 
        Npic_mask *nMlut, Npic_image *nCTg, Npic_l Rmax);

/* Backward compatibility <= 0.28 */
#define NpicExtractMA_Col NpicExtractMA_Mlut 


/* PRIVATE */

int NpicMlutIsMAlut_2l (Npic_image_2l *DT, Npic_mask_2l *Mlut, Npic_Lut *Lut,
        int ncol, int y, int x);
int NpicMlutIsMAlut_3l (Npic_image_3l *DT, Npic_mask_3l *Mlut, Npic_Lut *Lut,
        int ncol, int z, int y, int x);
int NpicMlutIsMAlut_4l (Npic_image_4l *DT, Npic_mask_4l *Mlut, Npic_Lut *Lut,
        int ncol, int t, int z, int y, int x);
int NpicMlutIsMAlut_5l (Npic_image_5l *DT, Npic_mask_5l *Mlut, Npic_Lut *Lut,
        int ncol, int s, int t, int z, int y, int x);
int NpicMlutIsMAlut_6l (Npic_image_6l *DT, Npic_mask_6l *Mlut, Npic_Lut *Lut,
        int ncol, int r, int s, int t, int z, int y, int x);

int NpicMlutIsMAcol_2l (Npic_image_2l *DT, Npic_mask_2l *Mlut, Npic_l *col,
        Npic_l Rmax, Npic_vec *vg, int y, int x);
int NpicMlutIsMAcol_3l (Npic_image_3l *DT, Npic_mask_3l *Mlut, Npic_l *col,
        Npic_l Rmax, Npic_vec *vg, int z, int y, int x);
int NpicMlutIsMAcol_4l (Npic_image_4l *DT, Npic_mask_4l *Mlut, Npic_l *col,
        Npic_l Rmax, Npic_vec *vg, int t, int z, int y, int x);
int NpicMlutIsMAcol_5l (Npic_image_5l *DT, Npic_mask_5l *Mlut, Npic_l *col,
        Npic_l Rmax, Npic_vec *vg, int s, int t, int z, int y, int x);
int NpicMlutIsMAcol_6l (Npic_image_6l *DT, Npic_mask_6l *Mlut, Npic_l *col,
        Npic_l Rmax, Npic_vec *vg, int r, int s, int t, int z, int y, int x);

int NpicMlutComp_L_WD (Npic_mask *nMg, Npic_l Rtarget);


#endif /* NPIC__MLUT_MA_H */
