/*
 * The Npic library
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This library is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/*
 * npic.h - 09/01/2003
 *
 * Main header of libNpic
*/


#ifndef NPIC__NPIC_H
#define NPIC__NPIC_H


/*
 * Standard headers
*/
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <limits.h>
#include <stdarg.h>
#include <math.h>

#include <unistd.h>
#include <memory.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>

/* MacOS */
#if defined(__APPLE_CC__) && !defined(__APPLE__)
# define __APPLE__
#endif

/* Endianness */
#ifdef __APPLE__
# include <machine/endian.h>
# define __LITTLE_ENDIAN __DARWIN_LITTLE_ENDIAN
# define __BIG_ENDIAN    __DARWIN_BIG_ENDIAN
# define __PDP_ENDIAN    __DARWIN_PDP_ENDIAN
# define __BYTE_ORDER    __DARWIN_BYTE_ORDER
#else
# include <endian.h>
#endif

/* OpenMP */
#ifdef NPIC_USE_OMP
# include <omp.h>
#endif


/*
 * Npic headers
*/

/* C++ compatibility */
#if defined(c_plusplus) && !defined(__cplusplus)
# define __cplusplus
#endif
#ifdef __cplusplus
extern "C" {
#endif

/* Fallback for __func__ */
#if __STDC_VERSION__ < 199901L
# if __GNUC__ >= 2
#  define __func__ __FUNCTION__
# else
#  define __func__ "<unknown>"
# endif
#endif

/* library headers */
#include "pixels.h"
#include "errors.h"
#include "misc.h"
#include "props.h"
#include "images.h"
#include "masks.h"

#include "image_creat.h"
#include "image_ntype.h"
#include "image_check.h"
#include "image_prop.h"
#include "image_border.h"
#include "image_copy.h"
#include "image_slice.h"

#include "mask_creat.h"
#include "mask_ntype.h"
#include "mask_check.h"
#include "mask_prop.h"
#include "mask_comp.h"
#include "mask_gsym.h"

#include "calc_values.h"
#include "calc_threshold.h"
#include "calc_algeb.h"
#include "calc_draw.h"
#include "calc_bresenham.h"
#include "calc_diff.h"

#include "dist_wdt.h"
#include "dist_sedt_hirata.h"
#include "dist_sedt_saito.h"
#include "dist_sedt_quadra.h"
#include "dist_dt.h"

#include "dist_skel_saito.h"

#include "mlut_remy.h"
#include "mlut_sedtg.h"
#include "mlut_wdtg.h"
#include "mlut_ma.h"

#include "files_hints.h"
#include "files_print.h"
#include "files_npz.h"
#include "files_pnm.h"
#include "files_vol.h"
#include "files_pan.h"
#include "files_raw.h"
#include "files_byext.h"
#include "files_gz.h"
#include "files_nmask.h"


/* C++ compatibility */
#ifdef __cplusplus
}
#endif


#endif /* NPIC__NPIC_H */

