/* 
    test-vol.c
    
    Edouard.Thiel@lif.univ-mrs.fr - 03/12/2007
    
    Test read/write of VOL files.
    
    Usage: ./test-vol
*/

#include <npic.h>


int main (int argc, char **argv)
{
    int x, y, z;
    Npic_image *np;
    Npic_image_3c *I;
    char *fname = "tmp.vol.gz";

    printf ("Creating 80*100*130 image ...\n");

    np = NpicCreateImage_3c (80, 100, 130, 3, 3, 3);
    if (np == NULL) exit (0);
    I = NpicCastImage (np);

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        I->pix[z][y][x] = (z*7+y*31+x*19) % 256;
    
    printf ("Adding properties ...\n");
    NpicImageAddProp (np, "Colour", "Blue and red : fine !");
    NpicImageAddProp (np, "Background", "White");
    
    printf ("Saving image .....\n");

    if (NpicWriteVOL (np, fname, 1) != NPIC_SUCCESS) exit (1);
    
    NpicDestroyImage (np);
    
    printf ("Loading image ....\n");

    np = NpicReadVOL (fname, 1);
    if (np == NULL) exit (1);
    I = NpicCastImage (np);

    printf ("Checking image ...\n");

    for (z = 0; z < I->zmax; z++)
    for (y = 0; y < I->ymax; y++)
    for (x = 0; x < I->xmax; x++)
        if (I->pix[z][y][x] != (z*7+y*31+x*19) % 256) {
            printf ("*** ERROR: bad value in %d %d %d : %d\n", 
                    x, y, z, I->pix[z][y][x]);
            NpicDestroyImage (np);
            exit (1);
        }
   
    printf ("SUCCESS.\n");
    
    printf ("Printing properties ...\n");
    NpicImagePrintProps (np);

    NpicDestroyImage (np);
 
    exit(0);
}

