/*
 * The Npic library and tools
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This program is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/*
 * npic-info.c - 18/10/2008
 *
*/

#include <npic.h>

void ShowUsage ()
{
    printf (
        "npic-info - Print info on image or mask file.\n"
        "Usage:\n"
        "  npic-info  -h | -help | --help       : print help\n"
        "  npic-info  in1                       : print infos on in1\n"
        "\n"
        "in1 : image or mask file\n"
        "\n"
        "Succeed if in1 can be read.\n"
    );
}

void ArgcExit (int argc, int n)
{
    if (argc < n) {
        fprintf (stderr, "ERROR: %d argument(s) missing, "
                         "type \"npic-info -h\" to get help.\n", n-argc);
        exit (1);
    }
}

void CheckResult (int k)
{
    switch (k) {
        case NPIC_SUCCESS : exit (0);
        case NPIC_ERR_BAD_EXT : break;
        default : exit (1);
    }
}

int main (int argc, char *argv[])
{
    char *in1;

    ArgcExit (argc, 2);
    
    if (strcmp (argv[1], "-h") == 0 || strcmp (argv[1], "-help") == 0 ||
        strcmp (argv[1], "--help") == 0) { ShowUsage (); exit (0); }
    
    in1 = argv[1];
    
    CheckResult (NpicInfoImage (in1));
    CheckResult (NpicInfoMask (in1));
        
    fprintf (stderr, 
        "### npic-info: bad extension for file \"%s\".\n"
        "    Known extensions are: "
        NPIC_KNOWN_IMAGE_EXT " " NPIC_KNOWN_MASK_EXT "\n", 
        argv[1]);

    exit (1);
}

