/*
 * The Npic library and tools
 *
 * Copyright (C) 2003 Edouard Thiel <Edouard.Thiel@lif.univ-mrs.fr>
 *
 * This program is free software under the terms of the 
 * GNU Lesser General Public License (LGPL) version 2.1.
*/

/*
 * npic-print.c - 22/09/2008
 *
*/

#include <npic.h>

void ShowUsage ()
{
    printf (
        "npic-print - Print image as text in console.\n"
        "Usage:\n"
        "  npic-print  -h | -help | --help       : print help\n"
        "  npic-print  in1                       : print in1\n"
        "\n"
        "in1 : image in " NPIC_KNOWN_IMAGE_EXT " format\n"
    );
}

void ArgcExit (int argc, int n)
{
    if (argc < n) {
        fprintf (stderr, "ERROR: %d argument(s) missing, "
                         "type \"npic-print -h\" to get help.\n", n-argc);
        exit (1);
    }
}

int main (int argc, char *argv[])
{
    char *in1;
    Npic_image *np1;

    ArgcExit (argc, 2);
    
    if (strcmp (argv[1], "-h") == 0 || strcmp (argv[1], "-help") == 0 ||
        strcmp (argv[1], "--help") == 0) { ShowUsage (); exit (0); }
    
    ArgcExit (argc, 2);
    in1  = argv[1];
    
    printf ("Loading \"%s\"\n", in1);
    np1 = NpicReadImage (in1);
    if (np1 == NULL) exit (1);
    
    NpicPrintImage (np1);
    
    NpicDestroyImage (np1);
    exit (0);
}

